@@ -1,4 +1,4 @@
-# copyright 2004-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2004-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of yams.
@@ -312,11 +312,12 @@ class X(Student):
     pass
 
 class Foo(B.EntityType):
-    i = B.Int(required=True)
+    i = B.Int(required=True, metadata={'name': B.String()})
     f = B.Float()
     d = B.Datetime()
 
 
+
 class PySchemaTC(TestCase):
 
     def test_python_inheritance(self):
@@ -336,10 +337,10 @@ class PySchemaTC(TestCase):
 
     def test_relationtype(self):
         foo = Foo()
-        self.assertEqual([r.etype for r in foo.__relations__],
-                          ['Int', 'Float', 'Datetime'])
+        self.assertEqual(['Int', 'String', 'Float', 'Datetime'],
+                         [r.etype for r in foo.__relations__])
         self.assertEqual(foo.__relations__[0].cardinality, '11')
-        self.assertEqual(foo.__relations__[1].cardinality, '?1')
+        self.assertEqual(foo.__relations__[2].cardinality, '?1')
 
     def test_maxsize(self):
         bp = BasePerson()
@@ -352,6 +353,11 @@ class PySchemaTC(TestCase):
         emp = Employee()
         self.assertEqual(maxsize(emp.__relations__[3]), 7)
 
+    def test_metadata(self):
+        foo = Foo()
+        self.assertEqual('i_name', foo.__relations__[1].name)
+
+
     def test_date_defaults(self):
         _today = date.today()
         _now = datetime.now()
@@ -382,6 +388,10 @@ class PySchemaTC(TestCase):
 
 
 class SchemaLoaderTC2(TestCase):
+
+    def tearDown(self):
+        SchemaLoader.main_schema_directory = 'schema'
+
     def test_broken_schema1(self):
         SchemaLoader.main_schema_directory = 'brokenschema1'
         with self.assertRaises(BadSchemaDefinition) as cm:
